/**
 * description
 *
 * @author eson
 *2022年7月21日-13:48:01
 */
package com.yuandian.dataflow.controller;

import java.io.Serializable;
import java.time.Instant;
import java.util.ArrayList;

import com.alipay.sofa.jraft.Status;
import com.alipay.sofa.jraft.entity.PeerId;
import com.alipay.sofa.jraft.error.RemotingException;
import com.alipay.sofa.jraft.rpc.InvokeCallback;
import com.alipay.sofa.jraft.rpc.RpcContext;
import com.alipay.sofa.jraft.rpc.RpcProcessor;
import com.google.protobuf.Any;
import com.yuandian.dataflow.statemachine.StateServerFactory;
import com.yuandian.dataflow.statemachine.operate.Operate;
import com.yuandian.dataflow.statemachine.operate.Operate.OperateType;
import com.yuandian.dataflow.statemachine.OperateClosure;
import com.yuandian.dataflow.statemachine.rpc.RaftResponse;
import com.yuandian.dataflow.statemachine.rpc.OperateProcessor;
 
import com.yuandian.dataflow.statemachine.rpc.annotations.ProcessorRaft;
import com.yuandian.dataflow.statemachine.state.State;

import lombok.Getter;
import lombok.Setter;
import lombok.var;
import lombok.extern.slf4j.Slf4j;

/**
 * description
 *
 * @author eson
 *2022年7月21日-13:48:01
 */

@Slf4j
@ProcessorRaft
public class PacketsProcessor implements RpcProcessor<PacketsProcessor.PacketsRequest> {

    @Setter
    @Getter
    public static class PacketsRequest implements Serializable {
        private ArrayList<Any> packets = new ArrayList<>();
    }

    @Override
    public void handleRequest(RpcContext rpcCtx, PacketsRequest request) {
            // StateServerFactory.getStateServer().updateFsmStateAsync(s, onCompleted);
            
            var resp = new RaftResponse();
            resp.setMsg(rpcCtx.getRemoteAddress());
           
 
            var ss = StateServerFactory.getStateServer();
 
            log.info("{} handler request.packets.size(): {}", StateServerFactory.getServerId(), request.packets.size());
            ss.readIndexState((state)->{
                
                var ws = state.getWorkers().get(StateServerFactory.getServerId());
                ws.setTaskQueueSize( ws.getTaskQueueSize() - request.packets.size());
                ws.setUpdateAt(Instant.now());

                log.debug("workerState taskQueueSize: {} psize: {} state {}", ws.getTaskQueueSize(), request.packets.size(), state.getWorkers().size());
               
                Operate.CallOperate(new Operate(OperateType.PUT,ws), new OperateClosure() {
                    @Override
                    public void run(Status status) {
                        var resp = this.getResponse();
                        if(status.isOk()) {
                            resp.setSuccess(true);
                            log.info("{}", resp);
                        } else {
                            resp.setSuccess(false);
                        }

                        rpcCtx.sendResponse(resp);
                    }
                });
            });
            

    }

    @Override
    public String interest() {
        return PacketsRequest.class.getName();
    }
}
